#ifndef __UNIFIED_APIS_H__
#define __UNIFIED_APIS_H__

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdint.h>
#include <json-c/json.h>
#include "kyee_security.h"
#include "tpm_security.h"

#define DEFAULT_KYTRUST_CONFIG          "/boot/efi/EFI/kylin/.libkytrusted-security.conf"
#define TRUSTED_MODE_KYEE               "1"
#define TRUSTED_MODE_TPM                "2"
#define TRUSTED_MODE_SOFT               "3" 
#define TRUSTED_MODE_TCM2               "5"

union TrustedBootMeasureData
{
    /*kyee 度量数据*/
    struct BootMeasureData kyee_bootMeasureData;
    /*tpm 度量数据*/
    struct TPMBootMeasureData tpm_bootMeasureData;
};


union TrustedBootMeasureLog
{
    /* kyee 历史度量数据 */
    struct BootMeasureLog kyee_bootMeasureLog;
    /* tpm 历史度量数据 */
    struct TPMBootMeasureLog tpm_bootMeasureLog;
};

union TrustChainData
{
    /* kyee 信任链数据 */
    struct KYEE_TrustChainData kyee_trustChainData;
    /* tpm 信任链数据 */
    struct TPM_TrustChainData tpm_trustChainData;
};

union Reportopt
{
    TpmReportOpt  tpm_opt;
    KyeeReportOpt kyee_opt;
};

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @brief 获取可信根数据
 * 
 * @param data 可信根数据的json格式
 * @param nvpasswd_len tpm nv保护口令的长度
 * @param nvpasswd tpm nv保护口令
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_GetTrustedRootData(char *data, int nvpasswd_len, char * nvpasswd);
/**
 * @brief 获取当前系统启动度量策略状态
 * 
 * @return -1， 失败；0 关闭； 1 初始化；2 开启； 3 警告；
 */
int TRUSTED_GetBootMeasureStatus();
/**
 * @brief 获取系统配置启动度量策略状态
 * 
 * @return 0 关闭； 1 初始化；2 开启； 3 警告； 其他 失败 
 */
int TRUSTED_GetBootMeasureCfgStatus();
/**
 * @brief 设置启动度量策略状态
 * 
 * @param status 度量策略状态， 0 关闭； 2 开启； 3 警告
 * @param devicepath 可信设备的路径（NULL为选用默认设备）
 * @param trustrootmode 返回当前可信模式（2）
 * @param nvpasswd_len tpm nv保护口令的长度
 * @param nvpasswd tpm nv保护口令
 * @return 0， 成功，-1， 失败
 */
int TRUSTED_SetBootMeasureCfgStatus(int status, char *devicepath, int *trustrootmode, int nvpasswd_len, char * nvpasswd , int modelue_status, char *modsname, int extern_status, int authorization_len, char * authorization_para);
/**
 * @brief 设置重新采集度量值
 * 
 * @param status 重新采集度量值 (0,不采集; 1,重新采集)
 * @param filepath 文件路径
 * @param nvpasswd_len tpm nv保护口令的长度
 * @param nvpasswd tpm nv保护口令
)
 * @return 0， 成功； -1， 失败
 */
int TRUSTED_SetReCollectMeasureStandarValue(int status,char *filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
/**
 * @brief 获取重新采集度量值 
 * 
 * @param filepath （all，全部度量； filepath，度量指定文件）
 * 
 * @return 0,不采集; 1,重新采集； -1， 失败 
 */
int TRUSTED_GetReCollectMeasureStandarValue();
/**
 * @brief 获取启动模式
 * 
 * @return  启动模式(0 同步； 1 混合；2 异步； 其他 失败)
 */
int TRUSTED_GetBootMode();
/**
 * @brief 设置启动模式
 * 
 * @param status 启动模式 ( 0 同步 1 混合 2 异步)
 * @return 0， 成功；其他失败
 */
int TRUSTED_SetBootMode(int status); 
/**
 * @brief flash启动日志存储
 * 
 * @return 0， 成功；其他失败 
 */
int TRUSTED_SaveFlashBootMeasureLog(); 

/**
 * @brief 获取当前启动报告
 * 
 * @param reportopt 需要获取的报告类型（1.bios 2.grub）
 * @param list 当前启动报告数据 
 * @param count 数据数量
 * @return 0， 成功；其他失败
 */
int TRUSTED_GetBootMeasureData(int reportopt, union TrustedBootMeasureData **list, int *count);
/**
 * @brief 获取历史启动报告
 * 
 * @param list 历史启动报告数据
 * @param count 数据数量
 * @return 0， 成功；其他失败 
 */
int TRUSTED_GetHistoryBootMeasureLog(union TrustedBootMeasureLog **list, int *count);

/**
 * @brief 获取当前使用的可信模式
 * 
 * @param trustMode 可信模式（1，kyee; 2, tpm; 3, soft）
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_GetCurrentUseTrustMode(char* trustMode);


//可信启动pks融合新增接口
/**
 * @brief 度量文件生成基准值并添加到.bimabase文件中（该功能暂不支持kyee可信根，如当前为kyee可信模式则报错）
 * 
 * @param filepath 文件路径  (in)   
 * @param nvpasswd_len tpm nv保护口令的长度
 * @param nvpasswd tpm nv保护口令
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_AddFile(char * filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);

/**
 * @brief 删除.bimabase文件中指定文件的度量信息（该功能不支持kyee可信根，如当前为kyee可信模式则报错）
 * 
 * @param filepath 文件路径  (in)   
 * @param nvpasswd_len tpm nv保护口令的长度
 * @param nvpasswd tpm nv保护口令
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_DealFile(char * filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);

/**
 * @brief获取可信链的度量结果
 * 
 * @param data 可信链数据  (out)   
 *
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_GetTrustChainData(union TrustChainData* data);

#ifdef __cplusplus
}
#endif

#endif //__UNIFIED_APIS_H__
